"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeIndexFeeds = void 0;
const fs = require("fs");
const path = require("path");
const os = require("os");
const util = require("util");
const DeferredPromise = require("p-defer");
const pify = util.promisify;
const rimraf = require('rimraf');
const pull = require('pull-stream');
const codec = require('flumecodec');
const Flume = require('flumedb');
const OffsetLog = require('flumelog-offset');
const { where, gt, toAsyncIter } = require('ssb-db2/operators');
const caps = require('ssb-caps');
const fromEvent = require('pull-stream-util/from-event');
const SecretStack = require('secret-stack');
const ssbKeys = require('ssb-keys');
const sample_1 = require("./sample");
function sampleAuthors(seed, authors, total) {
    const sampled = [];
    // Sample other authors, but don't sample ones that are already recipient
    while (sampled.length < total) {
        let other;
        // Keep generating new `other` until `sampled` is filled-up with unique ids
        do {
            other = (0, sample_1.paretoSample)(seed, authors).id;
        } while (sampled.some((r) => other === r));
        sampled.push(other);
    }
    return sampled
        .map((feedId) => authors.findIndex((author) => author.id === feedId))
        .filter((idx) => {
        if (idx < 0)
            throw new Error('sampleAuthors failed');
        else
            return true;
    });
}
function copyFlumelogOffset(origDir, destDir) {
    const flumeLogPath = path.join(origDir, 'flume', 'log.offset');
    fs.mkdirSync(path.join(destDir, 'flume'));
    fs.copyFileSync(flumeLogPath, path.join(destDir, 'flume', 'log.offset'));
}
function copySecret(origDir, destDir, i) {
    fs.copyFileSync(path.join(origDir, 'secret' + (i > 0 ? `-${i}` : '')), path.join(destDir, 'secret'));
}
function startSbot(dir) {
    const sbot = SecretStack({ caps })
        .use(require('ssb-db2'))
        .use(require('ssb-meta-feeds'))
        .use(require('ssb-index-feed-writer'))
        .call(null, {
        path: dir,
        keys: ssbKeys.loadOrCreateSync(path.join(dir, 'secret')),
        db2: {
            automigrate: true,
            dangerouslyKillFlumeWhenMigrated: true,
        },
    });
    // Patch `publishAs` so we signal (via debounce) when index writing is done
    const originalPublishAs = sbot.db.publishAs;
    const deferred = DeferredPromise();
    sbot._indexWritingComplete = deferred.promise;
    let timeout = null;
    sbot.db.publishAs = (...args) => {
        if (!timeout)
            timeout = setTimeout(deferred.resolve, 3000);
        timeout.refresh();
        originalPublishAs(...args);
    };
    return sbot;
}
function migrateDone(sbot) {
    return new Promise((resolve, reject) => {
        pull(fromEvent('ssb:db2:migrate:progress', sbot), pull.filter((x) => x === 1), pull.take(1), pull.collect((err) => {
            if (err)
                reject(err);
            else
                resolve(void 0);
        }));
    });
}
function openFlumedb(dir) {
    const flumeLogPath = path.join(dir, 'flume', 'log.offset');
    return Flume(OffsetLog(flumeLogPath, { codec: codec.json }));
}
function recentlyWrittenMsgs(timestamp, sbot) {
    return sbot.db.query(where(gt(timestamp, 'timestamp')), toAsyncIter());
}
function sanitizeMsg(msg) {
    if (msg.meta && msg.meta.originalContent) {
        msg.value.content = msg.meta.originalContent;
        delete msg.meta;
    }
    return msg;
}
function writeIndexFeeds(seed, indexFeedsPercentage, authors, followGraph, outputDir) {
    var e_1, _a;
    return __awaiter(this, void 0, void 0, function* () {
        // Pick which authors will write indexFeeds
        const totalIndexAuthors = Math.round((authors.length * indexFeedsPercentage) / 100);
        const sampledAuthorIdxs = sampleAuthors(seed, authors, totalIndexAuthors);
        // For each picked author:
        for (const i of sampledAuthorIdxs) {
            const lowestTimestamp = Date.now();
            const tempDir = fs.mkdtempSync(path.join(os.tmpdir(), 'index-feed-writer'));
            copyFlumelogOffset(outputDir, tempDir);
            copySecret(outputDir, tempDir, i);
            const sbot = startSbot(tempDir);
            yield migrateDone(sbot);
            const author = authors[i].id;
            for (const type of ['vote', 'post', 'contact', 'about']) {
                yield pify(sbot.indexFeedWriter.start)({ author, type, private: false });
            }
            yield pify(sbot.indexFeedWriter.start)({ author, type: null, private: true });
            yield sbot._indexWritingComplete;
            const flumedb = openFlumedb(outputDir);
            try {
                for (var _b = (e_1 = void 0, __asyncValues(recentlyWrittenMsgs(lowestTimestamp, sbot))), _c; _c = yield _b.next(), !_c.done;) {
                    const msg = _c.value;
                    const flumeMsg = sanitizeMsg(msg);
                    yield pify(flumedb.append)(flumeMsg);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            yield pify(flumedb.close)();
            if (followGraph) {
                // FIXME: update this object somehow
            }
            yield pify(sbot.close)();
            rimraf.sync(tempDir);
        }
    });
}
exports.writeIndexFeeds = writeIndexFeeds;
